/*
 * Copyright 2009 OW2 Chameleon
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ow2.chameleon.testing.tinybundles.ipojo;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.felix.ipojo.manipulator.Pojoization;
import org.apache.felix.ipojo.manipulator.Reporter;
import org.ops4j.pax.tinybundles.core.BuildStrategy;
import org.ops4j.pax.tinybundles.core.intern.BndBuilder;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.Map;

import static org.ops4j.pax.tinybundles.core.TinyBundles.withClassicBuilder;

/**
 * Tiny bundles extension to create iPOJO bundles.
 * @author <a href="mailto:chameleon-dev@ow2.org">Chameleon Project Team</a>
 */
public class IPOJOStrategy extends BndBuilder {

    private final File m_file;
    private Reporter reporter = new ConsoleReporter();

    public IPOJOStrategy(BuildStrategy builder) {
        super(builder);
        m_file = null;
    }

    public IPOJOStrategy(BuildStrategy builder, File metadata) {
        super(builder);
        m_file = metadata;
    }

    public InputStream build(Map<String, URL> resources, Map<String, String> headers) {
        InputStream stream = super.build(resources, headers);
        try {
            File tmp = File.createTempFile("tiny", ".jar");
            File tmp2 = File.createTempFile("pojo_out", ".jar");

            FileUtils.writeByteArrayToFile(tmp, IOUtils.toByteArray(stream));
            IOUtils.closeQuietly(stream);

            Pojoization pojoization = new Pojoization(reporter);
            pojoization.pojoization(tmp, tmp2, m_file);

            return FileUtils.openInputStream(tmp2);
        } catch (Exception e) {
            reporter.error("Really bad news", e);
        }
        return null;
    }

    public static BuildStrategy withiPOJO() {
        return new IPOJOStrategy(withClassicBuilder());
    }

    public static BuildStrategy withiPOJO(File metadata) {
        return new IPOJOStrategy(withClassicBuilder(), metadata);
    }

}
